VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CSizeGrip"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *************************************************************
'  Copyright 1999, Karl E. Peterson, All Rights Reserved.
'  http://www.mvps.org/vb/
' *************************************************************
'  Special thanks to Brad Martinez, who originated the concept.
'  http://members.aol.com/btmtz/vb/index.htm
' *************************************************************
'  Author grants royalty-free rights to use this code within
'  compiled applications. Selling or otherwise distributing
'  this source code is not allowed without author's express
'  permission.
' *************************************************************
Option Explicit

' Win32 API Declarations
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function InvalidateRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT, ByVal bErase As Long) As Long
Private Declare Function GetClientRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex As Long) As Long
Private Declare Function PtInRect Lib "user32" (lpRect As RECT, ByVal ptY As Long, ByVal ptX As Long) As Long
Private Declare Function GetDC Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, ByVal hDC As Long) As Long
Private Declare Function DrawFrameControl Lib "user32" (ByVal hDC As Long, lpRect As RECT, ByVal un1 As Long, ByVal un2 As Long) As Long
Private Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

' Flags for GetSystemMetrics
Private Const SM_CXSIZE = 30
Private Const SM_CYSIZE = 31

' Flags for DrawFrameControl
Private Const DFC_SCROLL = 3
Private Const DFCS_SCROLLSIZEGRIP = &H8

' Messages to hook
Private Const WM_PAINT = &HF
Private Const WM_NCHITTEST = &H84
Private Const WM_SIZE = &H5

' WM_NCHITTEST and MOUSEHOOKSTRUCT Mouse Position Codes
Private Const HTBOTTOMRIGHT = 17

' Member variables
Private m_hWnd As Long
Private m_ShowGrip As Boolean

' **************************************************************
'  Initialization/Termination
' **************************************************************
Private Sub Class_Initialize()
   ' Set default values
   m_ShowGrip = True
End Sub

Private Sub Class_Terminate()
   ' Always unhook!
   If m_hWnd Then
      Call UnhookWindow(m_hWnd)
   End If
End Sub

' **************************************************************
'  Public Properties
' **************************************************************
Public Property Let hWnd(ByVal NewVal As Long)
   ' Unhook previous window if need be.
   If m_hWnd Then
      Call UnhookWindow(m_hWnd)
   End If
   
   ' Store handle and hook new window.
   If IsWindow(NewVal) Then
      m_hWnd = NewVal
      Call HookWindow(m_hWnd, Me)
   End If
End Property

Public Property Get hWnd() As Long
   ' Return handle for window we're monitoring.
   hWnd = m_hWnd
End Property

Public Property Let ShowGrip(ByVal NewVal As Boolean)
   Dim rc As RECT
   
   ' Store new value, and force repaint of window.
   m_ShowGrip = NewVal
   If IsWindow(m_hWnd) Then
      Call GetClientRect(m_hWnd, rc)
      rc.Left = rc.Right - GetSystemMetrics(SM_CXSIZE)
      rc.Top = rc.Bottom - GetSystemMetrics(SM_CYSIZE)
      Call InvalidateRect(m_hWnd, rc, True)
   End If
End Property

Public Property Get ShowGrip() As Boolean
   ' Return property setting.
   ShowGrip = m_ShowGrip
End Property

' **************************************************************
'  Public Methods
' **************************************************************
Public Sub MakeSizable()
   If m_hWnd Then
      Call SetWindowStyleEx(m_hWnd, WS_EX_DLGMODALFRAME, False)
      Call SetWindowStyle(m_hWnd, WS_CLIPCHILDREN, True)
      Call SetWindowStyle(m_hWnd, WS_THICKFRAME, True, True)
   End If
End Sub

Public Sub MakeFixed()
   If m_hWnd Then
      Call SetWindowStyle(m_hWnd, WS_EX_DLGMODALFRAME, True)
      Call SetWindowStyle(m_hWnd, WS_CLIPCHILDREN, False)
      Call SetWindowStyle(m_hWnd, WS_THICKFRAME, False, True)
   End If
End Sub

' **************************************************************
'  Subclassing
' **************************************************************
Friend Function WindowProc(hWnd As Long, msg As Long, wp As Long, lp As Long) As Long
   Static hDC As Long
   Static rc1 As RECT
   Static rc2 As RECT
    
   If m_ShowGrip Then
      Select Case msg
         Case WM_SIZE
            ' Check if previous grip was drawn on visible
            ' surface. (Won't be the case if form was sized
            ' down very rapidly.) If so, just invalidate
            ' that region. If not, force a repaint.
            Call GetClientRect(hWnd, rc2)
            If PtInRect(rc2, rc1.Left, rc1.Top) Then
               Call InvalidateRect(hWnd, rc1, True)
            Else
               Call PostMessage(hWnd, WM_PAINT, 0, 0)
            End If
            
            ' Proceed with default processing.
            WindowProc = CallWindowProc(GetProp(hWnd, "OldWindowProc"), hWnd, msg, wp, lp)
            
         Case WM_PAINT
            ' Must allow the window to process the msg before
            ' the size grip is drawn, or things will get messed up...
            WindowProc = CallWindowProc(GetProp(hWnd, "OldWindowProc"), hWnd, msg, wp, lp)
            
            ' Draw a new size grip...
            Call GetClientRect(hWnd, rc1)
            rc1.Left = rc1.Right - GetSystemMetrics(SM_CXSIZE)
            rc1.Top = rc1.Bottom - GetSystemMetrics(SM_CYSIZE)
            hDC = GetDC(hWnd)
            Call DrawFrameControl(hDC, rc1, DFC_SCROLL, DFCS_SCROLLSIZEGRIP)
            Call ReleaseDC(hWnd, hDC)
         
         Case WM_NCHITTEST
            ' If the cursor is over the drawn size grip, have the window
            ' proc set the NW-SE pointer, allows the area to size the dlg.
            Call GetWindowRect(hWnd, rc2)
            rc2.Left = rc2.Right - GetSystemMetrics(SM_CXSIZE)
            rc2.Top = rc2.Bottom - GetSystemMetrics(SM_CYSIZE)
            If PtInRect(rc2, WordLo(lp), WordHi(lp)) Then
               WindowProc = HTBOTTOMRIGHT
            Else
               WindowProc = CallWindowProc(GetProp(hWnd, "OldWindowProc"), hWnd, msg, wp, lp)
            End If
         
         Case Else
            ' Insure all other messages are handled properly.
            WindowProc = CallWindowProc(GetProp(hWnd, "OldWindowProc"), hWnd, msg, wp, lp)
      End Select
   
   Else 'm_ShowGrip = False
      ' Allow default window message processing.
      WindowProc = CallWindowProc(GetProp(hWnd, "OldWindowProc"), hWnd, msg, wp, lp)
   End If
End Function

' **************************************************************
'  Private Methods
' **************************************************************
Private Function WordHi(LongIn As Long) As Integer
   Call CopyMem(WordHi, ByVal (VarPtr(LongIn) + 2), 2)
End Function

Private Function WordLo(LongIn As Long) As Integer
   Call CopyMem(WordLo, ByVal VarPtr(LongIn), 2)
End Function


