Attribute VB_Name = "MWindowStyles"
' *************************************************************
'  Copyright 1998-99, Karl E. Peterson, All Rights Reserved.
'  http://www.mvps.org/vb/
' *************************************************************
'  Author grants royalty-free rights to use this code within
'  compiled applications. Selling or otherwise distributing
'  this source code is not allowed without author's express
'  permission.
' *************************************************************
Option Explicit

Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, ByVal x As Long, ByVal y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long

Private Enum GWL_nIndex
   GWL_WNDPROC = (-4)
   GWL_HWNDPARENT = (-8)
   GWL_ID = (-12)
   GWL_STYLE = (-16)
   GWL_EXSTYLE = (-20)
   GWL_USERDATA = (-21)
End Enum

Private Enum SWP_Flags
   SWP_NOSIZE = &H1
   SWP_NOMOVE = &H2
   SWP_NOZORDER = &H4
   SWP_NOREDRAW = &H8
   SWP_NOACTIVATE = &H10
   SWP_FRAMECHANGED = &H20      ' The frame changed: send WM_NCCALCSIZE
   SWP_SHOWWINDOW = &H40
   SWP_HIDEWINDOW = &H80
   SWP_NOCOPYBITS = &H100
   SWP_NOOWNERZORDER = &H200    ' Don't do owner Z ordering
   SWP_NOSENDCHANGING = &H400   ' Don't send WM_WINDOWPOSCHANGING

   SWP_DRAWFRAME = SWP_FRAMECHANGED
   SWP_NOREPOSITION = SWP_NOOWNERZORDER

   SWP_DEFERERASE = &H2000
   SWP_ASYNCWINDOWPOS = &H4000
End Enum

Public Enum WinStyles
   WS_OVERLAPPED = &H0
   WS_TABSTOP = &H10000
   WS_MAXIMIZEBOX = &H10000
   WS_MINIMIZEBOX = &H20000
   WS_GROUP = &H20000
   WS_THICKFRAME = &H40000
   WS_SYSMENU = &H80000
   WS_HSCROLL = &H100000
   WS_VSCROLL = &H200000
   WS_DLGFRAME = &H400000
   WS_BORDER = &H800000
   WS_CAPTION = (WS_BORDER Or WS_DLGFRAME)
   WS_MAXIMIZE = &H1000000
   WS_CLIPCHILDREN = &H2000000
   WS_CLIPSIBLINGS = &H4000000
   WS_DISABLED = &H8000000
   WS_VISIBLE = &H10000000
   WS_MINIMIZE = &H20000000
   WS_CHILD = &H40000000
   WS_POPUP = &H80000000

   WS_TILED = WS_OVERLAPPED
   WS_ICONIC = WS_MINIMIZE
   WS_SIZEBOX = WS_THICKFRAME

   ' Common Window Styles
   WS_OVERLAPPEDWINDOW = (WS_OVERLAPPED Or WS_CAPTION Or WS_SYSMENU Or WS_THICKFRAME Or WS_MINIMIZEBOX Or WS_MAXIMIZEBOX)
   WS_TILEDWINDOW = WS_OVERLAPPEDWINDOW
   WS_POPUPWINDOW = (WS_POPUP Or WS_BORDER Or WS_SYSMENU)
   WS_CHILDWINDOW = WS_CHILD
End Enum   ' WinStyles

Public Enum WinStylesEx
   WS_EX_DLGMODALFRAME = &H1
   WS_EX_NOPARENTNOTIFY = &H4
   WS_EX_TOPMOST = &H8
   WS_EX_ACCEPTFILES = &H10
   WS_EX_TRANSPARENT = &H20

   WS_EX_MDICHILD = &H40
   WS_EX_TOOLWINDOW = &H80
   WS_EX_WINDOWEDGE = &H100
   WS_EX_CLIENTEDGE = &H200
   WS_EX_CONTEXTHELP = &H400

   WS_EX_RIGHT = &H1000
   WS_EX_LEFT = &H0
   WS_EX_RTLREADING = &H2000
   WS_EX_LTRREADING = &H0
   WS_EX_LEFTSCROLLBAR = &H4000
   WS_EX_RIGHTSCROLLBAR = &H0

   WS_EX_CONTROLPARENT = &H10000
   WS_EX_STATICEDGE = &H20000
   WS_EX_APPWINDOW = &H40000

   WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_CLIENTEDGE)
   WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE Or WS_EX_TOOLWINDOW Or WS_EX_TOPMOST)
End Enum   ' WinStylesEx


' Sets a windows style bits.
'   hWnd          - specified window's handle
'   dwNewStyle    - style to set
'   fAdd          - flag indicating whether the style is added or removed,
'                       If True, the style is added, otherwise the style is removed
'   fRedraw       - flag indicating whether to redraw the window so it detects the style change,
'                       If True, the window is redrawn, otherwise the window is not redrawn
'
' Returns True if the specified style is successfully added or removed.
' Returns False if either the specified style is already set, or on failure.

Public Function SetWindowStyle(hWnd As Long, dwNewStyle As WinStyles, fAdd As Boolean, Optional fRedraw As Boolean = False) As Boolean
   Dim dwCurStyle As Long
   Const dwStyleType As Long = GWL_STYLE

   dwCurStyle = GetWindowLong(hWnd, dwStyleType)
   If Err.LastDllError = 0 Then
      If fAdd And (dwCurStyle And dwNewStyle) = 0 Then
         ' Setting the new style and it is not already set...
         dwCurStyle = dwCurStyle Or dwNewStyle

      ElseIf (Not fAdd) And (dwCurStyle And dwNewStyle) Then
         ' Removing the new style and it's already set...
         dwCurStyle = dwCurStyle And (Not dwNewStyle)
      End If

      Call SetWindowLong(hWnd, dwStyleType, dwCurStyle)
      SetWindowStyle = (Err.LastDllError = 0)

      If fRedraw Then
         Call SetWindowPos(hWnd, 0, 0, 0, 0, 0, SWP_NOZORDER Or SWP_NOMOVE Or SWP_NOSIZE Or SWP_DRAWFRAME)
      End If
   End If
End Function

Public Function SetWindowStyleEx(hWnd As Long, dwNewStyle As WinStylesEx, fAdd As Boolean, Optional fRedraw As Boolean = False) As Boolean
   Dim dwCurStyle As Long
   Const dwStyleType As Long = GWL_EXSTYLE

   dwCurStyle = GetWindowLong(hWnd, dwStyleType)
   If Err.LastDllError = 0 Then
      If fAdd And (dwCurStyle And dwNewStyle) = 0 Then
         ' Setting the new style and it is not already set...
         dwCurStyle = dwCurStyle Or dwNewStyle

      ElseIf (Not fAdd) And (dwCurStyle And dwNewStyle) Then
         ' Removing the new style and it's already set...
         dwCurStyle = dwCurStyle And (Not dwNewStyle)
      End If

      Call SetWindowLong(hWnd, dwStyleType, dwCurStyle)
      SetWindowStyleEx = (Err.LastDllError = 0)

      If fRedraw Then
         Call SetWindowPos(hWnd, 0, 0, 0, 0, 0, SWP_NOZORDER Or SWP_NOMOVE Or SWP_NOSIZE Or SWP_DRAWFRAME)
      End If
   End If
End Function


